#!/usr/bin/env python

from .acfun import *
from .alive import *
from .archive import *
from .baidu import *
from .bandcamp import *
from .bigthink import *
from .bilibili import *
from .bokecc import *
from .cbs import *
from .ckplayer import *
from .cntv import *
from .coub import *
from .dailymotion import *
from .douban import *
from .douyin import *
from .douyutv import *
from .ehow import *
from .facebook import *
from .fc2video import *
from .flickr import *
from .freesound import *
from .funshion import *
from .google import *
from .heavymusic import *
from .icourses import *
from .ifeng import *
from .imgur import *
from .infoq import *
from .instagram import *
from .interest import *
from .iqilu import *
from .iqiyi import *
from .joy import *
from .ku6 import *
from .kugou import *
from .kuwo import *
from .le import *
from .lizhi import *
from .longzhu import *
from .magisto import *
from .metacafe import *
from .mgtv import *
from .miaopai import *
from .miomio import *
from .mixcloud import *
from .mtv81 import *
from .musicplayon import *
from .nanagogo import *
from .naver import *
from .netease import *
from .nicovideo import *
from .panda import *
from .pinterest import *
from .pixnet import *
from .pptv import *
from .qie import *
from .qingting import *
from .qq import *
from .showroom import *
from .sina import *
from .sohu import *
from .soundcloud import *
from .suntv import *
from .theplatform import *
from .tiktok import *
from .tucao import *
from .tudou import *
from .tumblr import *
from .twitter import *
from .ucas import *
from .veoh import *
from .videomega import *
from .vimeo import *
from .vine import *
from .vk import *
from .w56 import *
from .wanmen import *
from .xiami import *
from .yinyuetai import *
from .yixia import *
from .youku import *
from .youtube import *
from .ted import *
from .khan import *
from .zhanqi import *
from .kuaishou import *
from .zhibo import *
from .zhihu import *
